//
//  SnanerVC.swift
//  DontWaste
//
//  Created by David Aldorf on 03.03.18.
//  Copyright © 2018 David Aldorf. All rights reserved.
//

import UIKit
import RxCocoa
import RxSwift
import BarcodeScanner

class ScannerVC: UIViewController, RxViewControllerProtocol, StoryboardInit {
    
    @IBOutlet weak var nameTextField: UITextField!
    @IBOutlet weak var expirationDateFrom: UITextField!
    @IBOutlet weak var expirationDateTo: UITextField!
    @IBOutlet weak var barCodeTextField: UITextField!
    @IBOutlet weak var categoryTextField: UITextField!
    @IBOutlet weak var measureUnitTextField: UITextField!
    @IBOutlet weak var quantityTextField: UITextField!
    @IBOutlet weak var saveButton: RoundedButton!
    @IBOutlet weak var scanButton: UIButton!
    
    var expirationDateToDatePicker = UIDatePicker()
    var expirationDateFromDatePicker = UIDatePicker()
    
    let measureUnitPicker = UIPickerView()
    
    let categoryPicker = UIPickerView()
    
    internal var disposeBag = DisposeBag()
    
    var viewModel: ScannerVM?
    
    override func viewDidLoad() {
        
        super.viewDidLoad()
        
        setupUI()
        
        bindToVM()
    }
    
    private func setupUI() {
        
        if let product = viewModel?.product {
            
            nameTextField.text = product.name
            expirationDateFrom.text = product.expirationDateFrom?.string()
            expirationDateTo.text = product.expirationDateTo?.string()
            barCodeTextField.text = String(describing: product.barCode)
            categoryTextField.text = product.categoryString?.rawValue
            measureUnitTextField.text = product.measurementUnitString?.rawValue
            quantityTextField.text = String(describing: product.quantity)
            scanButton.isHidden = true
            saveButton.setTitle("Upravit", for: .normal)
        }
        
        //Formate Date
        expirationDateToDatePicker.datePickerMode = .date
        
        expirationDateTo.inputView = expirationDateToDatePicker
        
        expirationDateToDatePicker.addTarget(self, action: #selector(handleDatePickerTo(sender:)), for: .valueChanged)
        
        expirationDateFromDatePicker.datePickerMode = .date
        
        expirationDateFrom.inputView = expirationDateFromDatePicker
        
        expirationDateFromDatePicker.addTarget(self, action: #selector(handleDatePickerFrom(sender:)), for: .valueChanged)
        
        setupDatePicker(textField: expirationDateFrom)
        setupDatePicker(textField: expirationDateTo)
        setupDatePicker(textField: measureUnitTextField)
        setupDatePicker(textField: categoryTextField)
        
        measureUnitPicker.delegate = self
        measureUnitPicker.dataSource = self
        
        measureUnitTextField.inputView = measureUnitPicker
        
        categoryPicker.delegate = self
        categoryPicker.dataSource = self
        
        categoryTextField.inputView = categoryPicker
        
        let tapGesture = UITapGestureRecognizer(target: self, action: #selector(hideKeyboard))
        tapGesture.cancelsTouchesInView = true
        view.addGestureRecognizer(tapGesture)
    }
    
    internal func bindToVM() {
        
        viewModel?.navigator = Navigator(fridgeNavigationController: navigationController)
        
        let input = ScannerVM.Input(
            name: nameTextField.rx.text.orEmpty.asDriver(),
            expirationDateFrom: expirationDateFromDatePicker.rx.date.changed.asDriver(),
            expirationDateTo: expirationDateToDatePicker.rx.date.changed.asDriver(),
            barCode: barCodeTextField.rx.text.orEmpty.asDriver(),
            category: categoryTextField.rx.text.orEmpty.asDriver(),
            measureUnit: measureUnitTextField.rx.text.orEmpty.asDriver(),
            quantity: quantityTextField.rx.text.orEmpty.asDriver(),
            save: saveButton.rx.tap.asDriver()
        )
        
        let output = viewModel?.transform(input: input)
        
        [
            output?.save.drive()
        ].forEach({ $0?.disposed(by: disposeBag) })
    }
    
    func setupDatePicker(textField: UITextField) {
        
        //ToolBar
        let toolbar = UIToolbar();
        toolbar.sizeToFit()
        
        let doneButton = UIBarButtonItem(title: "Done", style: .done, target: self, action: #selector(hideKeyboard));
        toolbar.setItems([doneButton], animated: false)
        
        textField.inputAccessoryView = toolbar
    }
    
    @objc func handleDatePickerTo(sender: UIDatePicker) {
        
        expirationDateTo.text = sender.date.string()
    }
    
    @objc func handleDatePickerFrom(sender: UIDatePicker) {
        
        expirationDateFrom.text = sender.date.string()
    }
    
    @IBAction func touchUpInsideScanButton(_ sender: Any) {
        
        let barcodeScannerViewController = BarcodeScannerViewController()
        
        barcodeScannerViewController.codeDelegate = self
        barcodeScannerViewController.errorDelegate = self
        barcodeScannerViewController.dismissalDelegate = self
        
        present(barcodeScannerViewController, animated: true, completion: nil)
    }
    
    @objc func hideKeyboard() {
        
        view.endEditing(true)
    }
}

extension ScannerVC: BarcodeScannerCodeDelegate {
    func scanner(_ controller: BarcodeScannerViewController, didCaptureCode code: String, type: String) {
        barCodeTextField.text = code
        
        controller.dismiss(animated: true, completion: nil)
    }
}

extension ScannerVC: BarcodeScannerErrorDelegate {
    
    func scanner(_ controller: BarcodeScannerViewController, didReceiveError error: Error) {
        print(error)
    }
}

extension ScannerVC: BarcodeScannerDismissalDelegate {
    func scannerDidDismiss(_ controller: BarcodeScannerViewController) {
        controller.dismiss(animated: true, completion: nil)
    }
}

extension ScannerVC: UIPickerViewDataSource {
    func numberOfComponents(in pickerView: UIPickerView) -> Int {
        
        return 1
    }
    
    func pickerView(_ pickerView: UIPickerView, numberOfRowsInComponent component: Int) -> Int {
        
        if pickerView == measureUnitPicker {
            
            return MeasurementUnit.count
        } else if pickerView == categoryPicker {
            
            return Category.count
        }
        
        return 0
    }
}

extension ScannerVC: UIPickerViewDelegate {
    
    func pickerView(_ pickerView: UIPickerView, titleForRow row: Int, forComponent component: Int) -> String? {
        
        if pickerView == measureUnitPicker {
            
            return MeasurementUnit.datasource[row]
        } else if pickerView == categoryPicker {
            
            return Category.datasource[row]
        }
        
        return nil
    }
    
    func pickerView(_ pickerView: UIPickerView, didSelectRow row: Int, inComponent component: Int) {
        
        if pickerView == measureUnitPicker {
            
            measureUnitTextField.text = MeasurementUnit.datasource[row]
        } else if pickerView == categoryPicker {
            
            categoryTextField.text = Category.datasource[row]
        }
    }
}
