//
//  ScannerVM.swift
//  DontWaste
//
//  Created by David Aldorf on 03.03.18.
//  Copyright © 2018 David Aldorf. All rights reserved.
//

import RxSwift
import RxCocoa

class ScannerVM: ViewModelType {
    
    var product: Product?
    var navigator: Navigator?
    
    private let firebaseService: FirebaseService
    
    init(firebaseService: FirebaseService){
        
        self.firebaseService = firebaseService
    }
    
    struct Input {
        
        let name: Driver<String>
        let expirationDateFrom: Driver<Date>
        let expirationDateTo: Driver<Date>
        let barCode: Driver<String>
        let category: Driver<String>
        let measureUnit: Driver<String>
        let quantity: Driver<String>
        let save: Driver<Void>
    }
    
    struct Output {
        
        let save: Driver<Void>
    }
    
    func transform(input: ScannerVM.Input) -> ScannerVM.Output {
        
        let errorTracker = ErrorTracker()
        let activityIndicator = ActivityIndicator()
        
        let product = Driver.combineLatest(
            input.name,
            input.expirationDateFrom,
            input.expirationDateTo,
            input.barCode,
            input.category,
            input.measureUnit,
            input.quantity
            ).map { (arg) -> Product in
                
                let (name, expirationDateFrom, expirationDateTo, barCode, category, measureUnit, quantity) = arg
                
                return Product(
                    name: name,
                    expirationDateFrom: expirationDateFrom,
                    expirationDateTo: expirationDateTo,
                    barCode: Int(barCode) ?? 0000000000,
                    quantity: Int(quantity) ?? 0,
                    measurementUnitString: MeasurementUnit(rawValue: measureUnit) ?? MeasurementUnit.pieces,
                    categoryString: Category(rawValue: category) ?? Category.eggs
                )
        }
        
        let save = input.save.withLatestFrom(product).flatMapLatest { product -> SharedSequence<DriverSharingStrategy, Void> in
            
            guard let editProduct = self.product else {
                
                return self.firebaseService.createProduct(product: product)
                    .debug()
                    .trackActivity(activityIndicator)
                    .trackError(errorTracker)
                    .asDriverOnErrorJustComplete()
                    .do(onNext: self.navigator?.toProducts)
            }
            
            return self.firebaseService.updateProduct(product: product)
                .debug()
                .trackActivity(activityIndicator)
                .trackError(errorTracker)
                .asDriverOnErrorJustComplete()
                .do(onNext: self.navigator?.toProducts)
        }
        
        return ScannerVM.Output(save: save)
    }
}
