//
//  LoginVM.swift
//  DontWaste
//
//  Created by David Aldorf on 24.02.18.
//  Copyright © 2018 David Aldorf. All rights reserved.
//

import FacebookCore
import FirebaseAuth
import RxSwift
import RxCocoa
import SwiftKeychainWrapper

class LoginVM: ViewModelType {
    
    struct Input {
        let facebookLoginIn: Driver<Void>
        let loginWithEmail: Driver<Void>
        let email: Driver<String>
        let password: Driver<String>
    }
    
    struct Output {
        let facebookLoginOut: Driver<User?>
        let loginWithEmail: Driver<User?>
        let error: Driver<Error>
        let isEmailValid: Driver<Bool>
        let isPasswordValid: Driver<Bool>
        let fetching: Driver<Bool>
    }
    
    private let navigator = Navigator()
    
    private let facebookLoginService: FacebookLoginService
    private let firebaseAuthService: FirebaseAuthService

    init(facebookLoginService: FacebookLoginService, firebaseAuthService: FirebaseAuthService){
        
        self.facebookLoginService = facebookLoginService
        self.firebaseAuthService = firebaseAuthService
    }
    
    func transform(input: LoginVM.Input) -> LoginVM.Output {
        
        let errorTracker = ErrorTracker()
        let activityIndicator = ActivityIndicator()
        
        // MARK: - Validations
        
        let emailValidation = input.email
            .map { $0.isEmailValid }
            .debug()
            .trackError(errorTracker)
            .asDriverOnErrorJustComplete()
        
        let passwordValidation = input.password
            .map { $0.isPasswordValid }
            .debug()
            .trackError(errorTracker)
            .asDriverOnErrorJustComplete()
        
        // MARK: - Facebook Login
        
        let facebookLogin = input.facebookLoginIn
        .flatMapLatest {_ in
            
            self.facebookLoginService
                .facebookLogin()
                .debug()
                .trackError(errorTracker)
                .asDriverOnErrorJustComplete()
        }.flatMapLatest {_ in
            
            self.firebaseAuthService
                .signInWithFacebook()
                .debug()
                .trackActivity(activityIndicator)
                .trackError(errorTracker)
                .asDriverOnErrorJustComplete()
        }.do(onNext: { user in
            
            user?.getIDToken(completion: { (token, error) in
                
                guard
                    let token = token,
                    KeychainWrapper.standard.set(token, forKey: Constants.tokenKey)
                else {
                    
                    return
                }
                
                debugPrint(token)
            })
        
            self.navigator.createTabBar()
        })
        
        let credentials = Driver.combineLatest(input.email, input.password) {
            ($0, $1)
        }
        
        let loginWithEmail = input.loginWithEmail.withLatestFrom(credentials).flatMapLatest { (email, password) -> Driver<User?> in
            
            return self.firebaseAuthService.signIn(email: email, password: password)
                .debug()
                .trackActivity(activityIndicator)
                .trackError(errorTracker)
                .asDriver(onErrorRecover: { error in

                let err = error as NSError
                
                guard err.code == AuthErrorCode.userNotFound.rawValue else {
                    
                    return Driver.never()
                }
                
                return self.firebaseAuthService.createUser(email: email, password: password)
                    .debug()
                    .trackActivity(activityIndicator)
                    .trackError(errorTracker)
                    .asDriverOnErrorJustComplete()
                
            }).do(onNext: { _ in
                
                self.navigator.createTabBar()
            })
        }
        
        return LoginVM.Output(
            facebookLoginOut: facebookLogin,
            loginWithEmail: loginWithEmail,
            error: errorTracker.asDriver(),
            isEmailValid: emailValidation,
            isPasswordValid: passwordValidation,
            fetching: activityIndicator.asDriver()
        )
    }
    

}
