//
//  FirebaseService.swift
//  DontWaste
//
//  Created by David Aldorf on 03.03.18.
//  Copyright © 2018 David Aldorf. All rights reserved.
//

import Foundation
import Firebase
import FirebaseDatabase
import RxSwift
import RxCocoa
import ObjectMapper

protocol FirebaseServiceType: class {//: APIService {
    
}

extension Reactive where Base: DatabaseQuery {
    func observeSingleEventOfType(of eventType: DataEventType) -> Observable<DataSnapshot> {
        return Observable.create { observer in
            self.base.observeSingleEvent(of: eventType) { (snapshot) in
                observer.onNext(snapshot)
                observer.onCompleted()
            }
            
            return Disposables.create()
        }
    }
    
    func observe(eventType: DataEventType) -> Observable<DataSnapshot> {
        return Observable.create({ observer in
            let handle = self.base.observe(eventType, with: { snapshot in
                observer.onNext(snapshot)
            })
            
            return Disposables.create {
                self.base.removeObserver(withHandle: handle)
            }
        })
    }
}

extension Reactive where Base: DatabaseReference {
    func updateChildValues(values: [AnyHashable: Any]) -> Observable<DatabaseReference> {
        return Observable.create { observer in
            self.base.updateChildValues(values) { (error, reference) in
                if let error = error {
                    observer.onError(error)
                } else {
                    observer.onNext(reference)
                    observer.onCompleted()
                }
            }
            
            return Disposables.create()
        }
    }
    
    func setValue(value: Any?, andPriority priority: Any? = nil) -> Observable<DatabaseReference> {
        return Observable.create { observer in
            self.base.setValue(value, andPriority: priority) { (error, reference) in
                if let error = error {
                    observer.onError(error)
                } else {
                    observer.onNext(reference)
                    observer.onCompleted()
                }
            }
            
            return Disposables.create()
        }
    }
}

final class FirebaseService: FirebaseServiceType {
    private let database = Database.database().reference()

    lazy var products: Observable<[Product]> = {
        return getProducts()
    }()
    
    func getProducts() -> Observable<[Product]> {
        return observeValueArray(path: "\(Auth.auth().currentUser?.uid ?? "")").debug()
    }
    
    func createProduct(product: Product) -> Observable<Void> {
        let values = Mapper<Product>().toJSON(product)
        
        return database
            .child(Auth.auth().currentUser?.uid ?? "")
            .childByAutoId()
            .rx.updateChildValues(values: values)
            .mapToVoid()
    }
    
    func updateProduct(product: Product) -> Observable<Void> {
        guard let id = product.id else {
            return Observable.error(RxError.unknown)
        }
        
        let values = Mapper<Product>().toJSON(product)
        return database
            .child(Auth.auth().currentUser?.uid ?? "")
            .child(id)
            .rx.updateChildValues(values: values)
            .mapToVoid()
    }
    
    func getRecipes() -> Observable<[Recipe]> {
        return observeValueArray(path: "recipes").debug()
    }
    
    func createRecipe(recipe: Recipe) -> Observable<Void> {
        let values = Mapper<Recipe>().toJSON(recipe)
        
        return database
            .child("recipes")
            .childByAutoId()
            .rx.updateChildValues(values: values)
            .mapToVoid()
    }
    
    func updateRecipe(recipe: Recipe) -> Observable<Void> {
        guard let id = recipe.id else {
            return Observable.error(RxError.unknown)
        }
        
        let values = Mapper<Recipe>().toJSON(recipe)
        return database
            .child("recipes")
            .child(id)
            .rx.updateChildValues(values: values)
            .mapToVoid()
    }
    
    // MARK: Utilities
    private func observeValueArray<T: BaseMappable>(path: String) -> Observable<[T]> {
        return database
            .child(path)
            .rx.observe(eventType: .value)
            .map { Mapper<T>().mapArray(snapshot: $0) }
            .debug("observeValueArray")
    }
    
    private func observeSingleValueArray<T: BaseMappable>(path: String) -> Observable<[T]> {
        return database
            .child(path)
            .rx.observeSingleEventOfType(of: .value)
            .map({ snapshot in
                return Mapper<T>().mapArray(JSONObject: snapshot.value) ?? []
            })
            .share(replay: 1, scope: .forever)
    }
}
